﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using Greatis.Nostalgia;

namespace SystemCursorDemo
{
  public partial class FormMain : Form
  {
    private IntPtr curCur;
    private int hiddenCount;
    private IntPtr defaultCursor;

    public FormMain()
    {
      InitializeComponent();
      Font = SystemFonts.MessageBoxFont;
      winEvent.Active = true;
    }

    private void winEvent_ObjectNameChanged(object sender, Greatis.Nostalgia.ObjectEventArgs e)
    {
      IntPtr c = SystemCursor.GetCurrentCursor();
      if (e.ObjectID == ObjectType.Cursor && curCur != c)
      {
        curCur = c;
        panelCursor.Invalidate();
      }
    }

    private void panelNameChanged_Paint(object sender, PaintEventArgs e)
    {
      Control c = sender as Control;
      if (c != null && curCur != IntPtr.Zero)
      {
        using (Graphics g = Graphics.FromHwnd(c.Handle))
        {
          g.InterpolationMode = InterpolationMode.NearestNeighbor;
          using (Cursor cur = new Cursor(curCur))
          {
            using (Image img = new Bitmap(Cursor.Current.Size.Width, Cursor.Current.Size.Height))
            {
              using (Graphics ig = Graphics.FromImage(img))
              {
                cur.Draw(ig, new Rectangle(0, 0, img.Width, img.Height));
              }
              g.DrawImage(img, new Rectangle(0, 0, 128, 128));
            }
          }
        }
      }
    }

    private void UpdateButton()
    {
      buttonHideCursor.Text = string.Format("Cursor will be shown in {0} second(s)", hiddenCount);
    }

    private void buttonHideCursor_Click(object sender, EventArgs e)
    {
      hiddenCount = 5;
      UpdateButton();
      SystemCursor.Hide();
      timer.Enabled = true;
    }

    private void timer_Tick(object sender, EventArgs e)
    {
      if (hiddenCount > 1)
      {
        hiddenCount--;
        UpdateButton();
      }
      else
      {
        SystemCursor.Show();
        timer.Enabled = false;
        buttonHideCursor.Text = "Hide cursor";
      }
    }

    private void buttonReplace_Click(object sender, EventArgs e)
    {
      if (defaultCursor == IntPtr.Zero)
      {
        defaultCursor = SystemCursor.CopyCursor(SystemCursor.GetCursor(SystemCursors.Normal));
        SystemCursor.SetCursor(SystemCursors.Normal, new Cursor(GetType(), "Arrow.cur"));
        buttonReplace.Text = "Restore default arrow cursor";
        panelCursor.Invalidate();
      }
      else
      {
        SystemCursor.SetCursor(SystemCursors.Normal, defaultCursor);
        buttonReplace.Text = "Replace arrow cursor";
        defaultCursor = IntPtr.Zero;
        panelCursor.Invalidate();
      }
    }

    private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
    {
      SystemCursor.RestoreSystemCursors();
    }
  }
}
