﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using Greatis.Nostalgia;

namespace WinEventDemo
{
  public partial class FormMain : Form
  {
    private Point CurPos;
    private IntPtr CurCur;

    public FormMain()
    {
      InitializeComponent();
      Font = SystemFonts.MessageBoxFont;
      winEvent.Active = true;
    }

    private void winEvent_SystemAlert(object sender, Greatis.Nostalgia.SystemAlertEventArgs e)
    {
      ListViewItem lvi = listViewAlert.Items.Add(e.Alert.ToString());
      lvi.SubItems.Add(e.BoxHandle.ToString("X8"));
      lvi.SubItems.Add(e.BoxCaption);
      lvi.SubItems.Add(e.MessageText);
      lvi.SubItems.Add(e.Buttons.ToString());
      lvi.SubItems.Add(e.EventTime.ToString("X8"));
    }

    private void winEvent_WindowForeground(object sender, Greatis.Nostalgia.WindowEventArgs e)
    {
      ListViewItem lvi = listViewForeground.Items.Add(e.WindowHandle.ToString("X8"));
      SystemWindow sw = new SystemWindow(e.WindowHandle);
      lvi.SubItems.Add(sw.ClassName);
      lvi.SubItems.Add(sw.Text);
      lvi.SubItems.Add(e.EventTime.ToString("X8"));
    }

    private void AddMoveSize(IntPtr WindowHandle, uint EventTime, string Action)
    {
      ListViewItem lvi = listViewMoveSize.Items.Add(Action);
      lvi.SubItems.Add(WindowHandle.ToString("X8"));
      SystemWindow sw = new SystemWindow(WindowHandle);
      lvi.SubItems.Add(sw.Text);
      lvi.SubItems.Add("(" + sw.Left.ToString() + "," + sw.Top.ToString() + ")/(" + sw.Width.ToString() + "," + sw.Height.ToString() + ")");
      lvi.SubItems.Add(EventTime.ToString("X8"));
    }

    private void winEvent_MoveSizeStart(object sender, Greatis.Nostalgia.WindowEventArgs e)
    {
      AddMoveSize(e.WindowHandle, e.EventTime, "Start");
    }

    private void winEvent_MoveSizeEnd(object sender, Greatis.Nostalgia.WindowEventArgs e)
    {
      AddMoveSize(e.WindowHandle, e.EventTime, "End");
    }

    private void winEvent_ObjectLocationChanged(object sender, Greatis.Nostalgia.ObjectEventArgs e)
    {
      if (e.ObjectID == ObjectType.Cursor)
      {
        CurPos = Cursor.Position;
        labelCursorPosition.Text = string.Format("Cursor position ({0},{1})", CurPos.X, CurPos.Y);
        panelLocationChange.Invalidate();
      }
    }

    private void pictureBoxLocationChange_Paint(object sender, PaintEventArgs e)
    {
      Control c = sender as Control;
      if (c != null)
      {
        Point cp = c.PointToClient(CurPos);
        using (Graphics g = Graphics.FromHwnd(c.Handle))
        {
          using (Pen p = new Pen(Brushes.Orange, 3))
            g.DrawLine(p, new Point(c.Width / 2, c.Height / 2), cp);
        }
      }
    }

    private void winEvent_ObjectNameChanged(object sender, Greatis.Nostalgia.ObjectEventArgs e)
    {
      IntPtr c = SystemCursor.GetCurrentCursor();
      if (e.ObjectID == ObjectType.Cursor && CurCur != c)
      {
        labelNameChangeHandle.Text = string.Format("Current system cursor handle {0:X8}", (int)c);
        CurCur = c;
        panelNameChanged.Invalidate();
      }
    }

    private void panelNameChanged_Paint(object sender, PaintEventArgs e)
    {
      Control c = sender as Control;
      if (c != null)
      {
        using (Graphics g = Graphics.FromHwnd(c.Handle))
        {
          g.InterpolationMode = InterpolationMode.NearestNeighbor;
          using (Cursor cur = new Cursor(CurCur))
          {
            using (Image img = new Bitmap(Cursor.Current.Size.Width, Cursor.Current.Size.Height))
            {
              using (Graphics ig = Graphics.FromImage(img))
              {
                cur.Draw(ig, new Rectangle(0, 0, img.Width, img.Height));
              }
              g.DrawImage(img, new Rectangle(0, 0, 256, 256));
            }
          }
        }
      }
    }

    private void winEvent_SwitchStart(object sender, Greatis.Nostalgia.WindowEventArgs e)
    {
      Size s = Screen.PrimaryScreen.Bounds.Size;
      SystemWindow sw = new SystemWindow(e.WindowHandle);
      if (radioButtonAltTabLeftTop.Checked) sw.Location = new Point(0, 0);
      else
        if (radioButtonAltTabLeftBottom.Checked) sw.Location = new Point(0, s.Height - sw.Height);
        else
          if (radioButtonAltTabRightTop.Checked) sw.Location = new Point(s.Width - sw.Width, 0);
          else
            if (radioButtonAltTabRightBottom.Checked) sw.Location = new Point(s.Width - sw.Width, s.Height - sw.Height);
    }

    private void winEvent_WinEventHook(object sender, IntPtr WinEventHook, uint EventType, IntPtr WindowHandle, int ObjectID, int ChildID, uint EventThread, uint EventTime, ref bool Enabled)
    {

    }

  }
}
