﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Greatis.Nostalgia;

namespace WindowInformationDemo
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
      Font = SystemFonts.MessageBoxFont;
      radioGroupKind.UpdateButtons();
      radioGroupWindowState.UpdateButtons();
    }

    private void windowInformation_WindowAdded(object sender, Greatis.Nostalgia.SystemWindowEventArgs e)
    {
      ListViewItem lvi = listViewWindows.Items.Add(e.Window.ClassName, ((int)e.Window.WindowKind) - 1);
      lvi.SubItems.Add(e.Window.Text);
      lvi.SubItems.Add(e.Window.Left.ToString());
      lvi.SubItems.Add(e.Window.Top.ToString());
      lvi.SubItems.Add(e.Window.Width.ToString());
      lvi.SubItems.Add(e.Window.Height.ToString());
      lvi.Tag = e.Window;
    }

    private void buttonFind_Click(object sender, EventArgs e)
    {
      listViewWindows.Items.Clear();
      string sclass = null;
      if (checkBoxClassName.Checked) sclass = textBoxClassNameSearch.Text;
      string stext = null;
      if (checkBoxText.Checked) stext = textBoxTextSearch.Text;
      windowInformation.SearchWindows((WindowKind)radioGroupKind.ItemIndex, sclass, stext, checkBoxExact.Checked);
      labelResult.Text = string.Format("{0} window(s) found", windowInformation.Windows.Count);
    }

    private void AddDetail(string name, uint value)
    {
      ListViewItem lvi = listViewDetails.Items.Add(name);
      lvi.SubItems.Add(value.ToString("X8"));
    }

    private void listViewWindows_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
    {
      if (listViewWindows.SelectedItems.Count > 0)
      {
        listViewDetails.Items.Clear();
        SystemWindow sw = (SystemWindow)(listViewWindows.SelectedItems[0].Tag);
        if (sw != null)
        {
          AddDetail("Handle", (uint)sw.Handle);
          AddDetail("ResourceID", (uint)sw.ResourceID);
          AddDetail("ProcessID", (uint)sw.ProcessID);
          AddDetail("AppInstance", (uint)sw.AppInstance);
          AddDetail("Parent", (uint)sw.Parent);
          AddDetail("ControlID", (uint)sw.ControlID);
          AddDetail("Function", (uint)sw.Function);
          AddDetail("Menu", (uint)sw.Menu);
          AddDetail("Style", (uint)sw.Style);
          AddDetail("ExStyle", (uint)sw.ExStyle);
          AddDetail("ClassFunction", (uint)sw.ClassFunction);
          AddDetail("ClassIcon", (uint)sw.ClassIcon);
          AddDetail("ClassSmallIcon", (uint)sw.ClassSmallIcon);
          AddDetail("ClassCursor", (uint)sw.ClassCursor);
          AddDetail("ClassBrush", (uint)sw.ClassBrush);
          AddDetail("ClassStyle", (uint)sw.ClassStyle);

          UpdateEditBox(sw);
        }
      }
    }

    private void UpdateEditBox(SystemWindow sw)
    {
      radioGroupWindowState.ItemIndex = (int)sw.WindowState;
      checkBoxVisible.Checked = sw.Visible;
      checkBoxEnabled.Checked = sw.Enabled;
      checkBoxStayOnTop.Checked = sw.StayOnTop;
      textBoxLeft.Text = sw.Left.ToString();
      textBoxTop.Text = sw.Top.ToString();
      textBoxWidth.Text = sw.Width.ToString();
      textBoxHeight.Text = sw.Height.ToString();
      textBoxText.Text = sw.Text;
    }

    private void checkBoxEnableEdit_CheckedChanged(object sender, EventArgs e)
    {
      if (checkBoxEnableEdit.Checked)
        if (MessageBox.Show("Changing of some windows can be dangerous for your system." + Environment.NewLine + "Enable edit anyway?", "Please be careful", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) != DialogResult.Yes) checkBoxEnableEdit.Checked = false;
      groupBoxEdit.Enabled = checkBoxEnableEdit.Checked;
    }

    private void buttonReset_Click(object sender, EventArgs e)
    {
      SystemWindow sw = (SystemWindow)(listViewWindows.SelectedItems[0].Tag);
      if (sw != null) UpdateEditBox(sw);
    }

    private void buttonApply_Click(object sender, EventArgs e)
    {
      SystemWindow sw = (SystemWindow)(listViewWindows.SelectedItems[0].Tag);
      if (sw != null)
      {
        sw.WindowState = (FormWindowState)radioGroupWindowState.ItemIndex;
        sw.Visible = checkBoxVisible.Checked;
        sw.Enabled = checkBoxEnabled.Checked;
        sw.StayOnTop = checkBoxStayOnTop.Checked;
        sw.Text = textBoxText.Text;

        int x;
        if (int.TryParse(textBoxLeft.Text, out x)) sw.Left = x;
        if (int.TryParse(textBoxTop.Text, out x)) sw.Top = x;
        if (int.TryParse(textBoxWidth.Text, out x)) sw.Width = x;
        if (int.TryParse(textBoxHeight.Text, out x)) sw.Height = x;
      }
    }

  }
}
